function edit.to_loc(Editor, mx,my)
  if my < Editor.top then
    return deepcopy(Editor.screen_top)
  end
  local y = Editor.top
  for line_index,line in array.each(Editor.lines, Editor.screen_top.line) do
    if line.mode == 'text' then
      local x = Editor.left
      local initpos = 1
      if line_index == Editor.screen_top.line then
        initpos = Editor.screen_top.pos
      end
      for pos, char in utf8chars(line.data, initpos) do
        local w = Editor.font:getWidth(char)  -- width of char
        if char:match('%s') then
          if Text.line_wrap_at_word_boundary(Editor, x, line.data, pos) then
            if my < y+Editor.line_height then
              return {mode='text', line=line_index, pos=pos}
            end
            x = Editor.left
            y = y + Editor.line_height
          else
            if my < y+Editor.line_height and mx < x+w then
              return {mode='text', line=line_index, pos=pos}
            end
            x = x + w
          end
        else
          if x+w > Editor.right then
            if my < y+Editor.line_height then
              return {mode='text', line=line_index, pos=pos}
            end
            x = Editor.left
            y = y + Editor.line_height
            if my < y+Editor.line_height and mx < x+w then
              return {mode='text', line=line_index, pos=pos}
            end
          else
            if my < y+Editor.line_height and mx < x+w then
              return {mode='text', line=line_index, pos=pos}
            end
          end
          x = x+w
        end
      end
      if my < y+Editor.line_height then
        return {mode='text', line=line_index, pos=utf8.len(line.data)+1}
      end
      y = y + Editor.line_height
    else
      -- drawing
      local h = Drawing_padding_height + Drawing.pixels(line.h, Editor.width)
      if my < y+h then
        return {mode='drawing', line=line_index}
      end
      y = y + h
    end
    if y + Editor.line_height > Editor.bottom then
      break
    end
  end
end

function edit.to_coord(Editor, loc)  -- scans
  local y = Editor.top
  for line_index, line in array.each(Editor.lines, Editor.screen_top.line) do
    if line.mode == 'text' then
      local x = Editor.left
      local initpos = 1
      if line_index == Editor.screen_top.line then
        initpos = Editor.screen_top.pos
      end
      for pos,char in utf8chars(line.data, initpos) do
        local w = Editor.font:getWidth(char)
        if char:match('%s') then
          if Text.line_wrap_at_word_boundary(Editor, x, line.data, pos) then
            if loc.mode == 'text' and loc.line == line_index and loc.pos == pos then
              return x, y
            end
            x = Editor.left
            y = y + Editor.line_height
          else
            if loc.mode == 'text' and loc.line == line_index and loc.pos == pos then
              return x, y
            end
            x = x + w
          end
        else
          if x+w > Editor.right then
            x = Editor.left
            y = y + Editor.line_height
            if loc.mode == 'text' and loc.line == line_index and loc.pos == pos then
              return x, y
            end
          else
            if loc.mode == 'text' and loc.line == line_index and loc.pos == pos then
              return x, y
            end
          end
          x = x + w
        end
      end
      if loc.mode == 'text' and loc.line == line_index and loc.pos == utf8.len(line.data)+1 then
        return x, y
      end
      y = y + Editor.line_height
    elseif line.mode == 'drawing' then
      if loc.mode == 'drawing' and loc.line == line_index then
        return Editor.left, y
      end
      y = y + Drawing.height(Editor, line)
    else
      assert(false, ('unknown line mode %s'):format(line.mode))
    end
    if y + Editor.line_height > Editor.bottom then
      break
    end
  end
end  -- 1 scan

-- find the location at the start of a screen line dy pixels down from loc
-- return nil if dy is more than screen height away
-- return bottommost screen line in file if we hit it
function edit.down(Editor, loc, dy)  -- scans
  local y = 0
  local prevloc = loc
  for line_index, line in array.each(Editor.lines, loc.line) do
    if line.mode == 'text' then
      local screen_line_starts = Text.screen_line_starts(Editor, line_index)  -- scan
      local start_counting_screen_line_index = 1
      if line_index == loc.line then
        start_counting_screen_line_index = Text.start_of_screen_line(Editor, loc)  -- extra scan for just one line
      end
      for screen_line_index = start_counting_screen_line_index,#screen_line_starts do
        local currloc = {mode='text', line=line_index, pos=screen_line_starts[screen_line_index]}
        if y+Editor.line_height > dy then
          return currloc
        end
        y = y + Editor.line_height
        prevloc = currloc
      end
    elseif line.mode == 'drawing' then
      local currloc = {mode='drawing', line=line_index}
      local h = Drawing.height(Editor, line)
      if y + h > dy then
        return currloc
      end
      y = y + h
      prevloc = currloc
    else
      assert(false, ('unknown line mode %s'):format(line.mode))
    end
    if y + Editor.line_height > Editor.bottom - Editor.top then
      return nil
    end
  end
  return prevloc
end  -- 1 scan

-- find the location at the start of a screen line dy pixels up from loc
-- return nil if dy is more than screen height away
-- return topmost screen line in file if we hit it
function edit.up(Editor, loc, dy)  -- scans
  local y = 0
  for line_index = loc.line,1,-1 do
    local line = Editor.lines[line_index]
    if line.mode == 'text' then
      local screen_line_starts = Text.screen_line_starts(Editor, line_index)  -- scan
      local ignore_below_screen_line_index = #screen_line_starts+1
      if line_index == loc.line then
        ignore_below_screen_line_index = Text.start_of_screen_line(Editor, loc)  -- extra scan for just one line
      end
      for screen_line_index = ignore_below_screen_line_index-1,1,-1 do
        if y+Editor.line_height > dy then
          return {mode='text', line=line_index, pos=screen_line_starts[screen_line_index]}
        end
        y = y + Editor.line_height
      end
    elseif line.mode == 'drawing' then
      local h = Drawing.height(Editor, line)
      if line_index == loc.line then
        -- ignore the current loc entirely if it's a drawing
        h = 0
      end
      if y + h > dy then
        return {mode='drawing', line=line_index}
      end
      y = y + h
    else
      assert(false, ('unknown line mode %s'):format(line.mode))
    end
    if y + Editor.line_height > Editor.bottom - Editor.top then
      return nil
    end
  end
  return edit.top(Editor)
end  -- 1 scan
