function edit.down(Editor, loc, dy)
  local line_rects = edit.line_rects()
  local cx, cy = edit.to_coord(Editor, line_rects, loc)
  if cx then
    return edit.to_loc(Editor, line_rects, cx, cy + dy)
  else
    -- TODO: handle the location not being valid somehow?
  end
end

function edit.up(Editor, loc, dy)
  local line_rects = edit.line_rects()
  local cx, cy = edit.to_coord(Editor, line_rects, loc)
  if cx then
    return edit.to_loc(Editor, line_rects, cx, cy - dy)
  else
    -- TODO: handle the location not being valid somehow?
  end
end

function edit.to_loc(Editor, line_rects, mx,my)
  if my < Editor.top then
    return deepcopy(Editor.screen_top)
  end
  local y = Editor.top

  local file_line_rect = line_rects.get_under(mx, my)
  local line = file_line_rect.value

  if line.mode = 'text' then
    local screen_line = line.screen_lines.get_under(mx, my)
    -- TODO: get_char_under
    local _, char_pos = screen_line:get_char_under(mx,my)
    return {mode='text', index=line.index, pos=char_pos}
  elseif line.mode = 'drawing' then
    return {mode='drawing', index=line.index}
  else
    return nil -- signal out of bounds click?
  end
end

function edit.to_coord(Editor, line_rects, loc)  -- scans
    local r = line_rects.at(loc.index)
    if loc.mode == 'drawing' then
        return r.x, r.y
    elseif loc.mod == 'text' then
        return char_coord(r.value.line, loc.pos)
    end
end

function char_coord(Editor, x, y, file_line, pos)
  local screen_lines = Text.get_screen_lines(file_line.data, Editor.width)
  local seek = 0
  local cx, cy = x, y
  for line_idx, line in ipairs(screen_lines) do
    for cpos, char in utf8chars(line) do
      seek = seek + 1
      if seek = pos then
        return cx , cy
      end
      cx = cx + Editor.font:getWidth(char)
    end
    cx = x
    cy = cy + Editor.line_height
  end
  return nil
end

function edit.line_rects(Editor)
  local rects = {}
  local y = Editor.top
  for line_indx, line in array.each(Editor.lines, Editor.screen_top.line) do
    if line.mode == 'text' then
      -- TODO: Deal with wrapped text by building the outer rect
      -- based on line-wrapping
      local lines, h = get_wrapped_line_rects(Editor, Editor.left, y, line)
      local rect_value = {index=line_index, mode='text', line=line, screen_lines=lines}
      table.insert(rects, Rect(Editor.left, w, y, h, ret_value))
      y = y + h
    elseif line.mode == 'drawing' then
      local h = Drawing.height(line)
      table.insert(rects, Rect(Editor.left, Editor.width, y, h, {index=line_index, mode='drawing', line=line}))
      y = y + h
    end
  end

  return RectList(rects)
end

function get_wrapped_line_rects(Editor, x, y, file_line)
  local screen_lines = Text.get_screen_lines(file_line.data, Editor.width)
  local line_rects = {}
  for line_idx, line in ipairs(screen_lines) do
    local rect = Rect(
      x,
      y + Editor.line_height * line_idx,
      Editor.font:getWidth(line), -- not sure of the shape here
      Editor.line_height,
      line)
    table.insert(line_rect, rect)
  end

  return RectList(line_rects), #line_rects * Editor.line_height
end

function RectList(rects)
    local me = {}

    function me.at(idx) return rects[idx] end

    function me.get_under(x,y)
      for i, rect in ipairs(rects) do
        if rect:contains(x, y) then return rect end
      end
      return nil
    end

    return me
end

function Rect(x, y, w, h, value)
  return {
    x=x, y=y, w=w, h=h,
    bottom = y+h,
    right = x+w,
    value = value,
    contains = function (r, x, y)
      return x > r.x and y > r.y and x < r.right and y < r.bottom
    end,
  }
end

-- assume line line_index starts at y
-- generate rects for each screen line in it
function Text.get_screen_line_rects(Editor, line_index, y)
  local result = {}
  local line = Editor.lines[line_index]
  assert(line.mode == 'text')
  local s = 1
  local x = Editor.left
  for pos,char in utf8chars(line.data) do
    local w = Editor.font:getWidth(char)
    if char:match('%s') then
      if Text.line_wrap_at_word_boundary(Editor, x, line.data, pos) then
        table.insert(result,
          {x=Editor.left, y=y, w=Editor.width, h=Editor.line_height,
           s=s, e=pos})
        s = pos+1
        x = Editor.left
        y = y + Editor.line_height
      else
        -- nothing
        x = x + w
      end
    else
      if x+w > Editor.right then
        table.insert(result,
          {x=Editor.left, y=y, w=Editor.width, h=Editor.line_height,
           s=s, e=pos-1})
        x = Editor.left
        y = y + Editor.line_height
        s = pos
      else
        -- nothing
      end
      x = x + w
    end
  end
  if #result == 0 or #curr_screen_line > 0 then
    table.insert(result,
      {x=Editor.left, y=y, w=Editor.width, h=Editor.line_height, s=s, e=utf8.len(line.data)})
  end
  return result
end
