-- This is a subset of the file in question, not everything

function rect_contains(r, x, y)
    return x > r.x and y > r.y and x < r.right and y < r.bottom
end

function Rect(x, y, w, h, value)
    local me = {
        x=x,y=y,w=w,h=h
        bottom = y+h, 
        right = x+w,
        value = value,
    }

    me.contains = rect_contains
    return me
end

function RectList(rects) 
    local me = {}

    function me.at(idx) return rects[idx] end
    function me.get_under(x,y)
        for i, rect in ipairs(rects) do
            if rect:contains(x, y) then return rect end
        end
        return nil
    end

    return me
end

function char_coord(Editor, x, y, file_line, pos)
    local lines, w = Text.get_wrap(file_line.data, Editor.width)
    local seek = 0
    local cx, cy = x, y
    for line_idx, line in ipairs lines do
        for cpos, char utf8chars(line) do
            seek = seek + 1
            if seek = pos then
                return cx , cy
            end
            cx = cx + Editor.font:getWidth(char)
        end
        cx = x
        cy = cy + Editor.line_height
    end
    return nil
end

function get_wrapped_line_rects(Editor, x, y, file_line)
    local lines, w = Text.get_wrap(file_line.data, Editor.width)
    local line_rects = {}
    for line_idx, line in ipairs(lines) do
        table.insert(line_rect, 
            Rect(x, 
                y + Editor.line_height * line_idx,
                Editor.line_height,
                Editor.font:getWidth(line), -- not sure of the shape here
                line)
        )
    end

    return RectList(line_rects), #line_rects * Editor.line_height
end

function edit.line_rects(Editor)
    local rects = {}
    local y = Editor.top
    for line_indx, line in array.each(Editor.lines, Editor.screen_top.line) do
        if line.mode == 'text' then
            -- TODO: Deal with wrapped text by builduing the outer rect
            -- based on line-wrapping
            local lines, h = get_wrapped_line_rects(Editor, Editor.left, y, line)
            table.insert(rects, Rect(Editor.left, w, y, h, {
                index = line_index, 
                mode='text',
                line = line,
                screen_lines = lines }))
            y = y + h
        elseif line.mode == 'drawing' then
            local h = Drawing.height(line)
            table.insert(rects, Rect(Editor.left, Editor.width, y, h, {index=line_index, mode='drawing', line=line}))
            y = y + h
        end
    end

    return RectList(rects)
end


function edit.to_loc(Editor, line_rects, mx,my)
if my < Editor.top then
    return deepcopy(Editor.screen_top)
    end
    local y = Editor.top

    local file_line_rect = line_rects.get_under(mx, my)
    local line = file_line_rect.value

    if line.mode = 'text' then
      local screen_line = line.screen_lines:get_under(mx, my)
      -- get_char_under not built, char_pos would be the position in the file_line, as I understand it
      local char, char_pos = screen_line:get_char_under(mx,my) 
      return {mode='text', index=line.index, pos=char_pos}
    elseif line.mode = 'drawing' then
      return line -- might be enough?
    else
      return nil -- signal out of bounds click?
    end
end

function edit.to_coord(Editor, rects, loc)  -- scans
    local r = rects.at(loc.index)
    if loc.mode == 'drawing' then
        return r.x, r.y
    elseif loc.mod == 'text' then
        return char_coord(r.value.line, loc.pos)
    end
end

function edit.down(Editor, loc, dy)
    local line_rects = edit.line_rects()
    local cx, cy = edit.to_coord(Editor, line_rects, loc)
    if cx then
        return edit.to_loc(Editor, line_rects, cx, cy + dy)
    else
        -- handle the location not being valid somehow?
    end
end

function edit.up(Editor, loc, dy)
    local line_rects = edit.line_rects()
    local cx, cy = edit.to_coord(Editor, line_rects, loc)
    if cx then
        return edit.to_loc(Editor, line_rects, cx, cy - dy)
    else
        -- handle the location not being valid somehow?
    end
end
